<?php

$RAD_TO_DEG = 180/M_PI;
$DEG_TO_RAD = 1/$RAD_TO_DEG;
$NM_TO_M = 1852;
$M_TO_NM = 1 / $NM_TO_M;
$EARTH_RADIUS_IN_METERS = 6372797.560856;
$ARC_GRANULARITY = 0.1; //M_PI/10;


function distToAngle($dist) {
  global $EARTH_RADIUS_IN_METERS;
  return rad2deg( asin($dist/$EARTH_RADIUS_IN_METERS) );
}

/*
function distToAngle($distInNauticalMiles) {
  global $NM_TO_M, $EARTH_RADIUS_IN_METERS, $RAD_TO_DEG;
  $dist = $NM_TO_M * $distInNauticalMiles;
  $angleRad = asin($dist/$EARTH_RADIUS_IN_METERS);
  $angleDeg = $angleRad * $RAD_TO_DEG;
  return $angleDeg;
}
*/


function computeArcPoint($clat, $clon, $r, $partAngle) {
  global $DEG_TO_RAD;
  $lon = sin($partAngle) * $r;
  $lat = cos($partAngle) * $r;


  $rlat = $clat + $lat;
  $rlon = $clon + $lon / cos ($rlat*$DEG_TO_RAD);
  return array('lat' => $rlat, 'lon' => $rlon);
}


function insertCircle($pos, $radius) {
  global $ARC_GRANULARITY;

  $r = distToAngle($radius);

  $clat = $pos['lat'];
  $clon = $pos['lon'];

  $result = array();
  for ($k = 0; $k < 2*M_PI; $k += $ARC_GRANULARITY) {
    array_push($result, computeArcPoint($clat, $clon, $r, $k));
  }

  return $result;
}

function arcInRadians($from, $to) {
  global $DEG_TO_RAD;
  $latitudeArc  = ($from['lat'] - $to['lat']) * $DEG_TO_RAD;
  $longitudeArc = ($from['lon'] - $to['lon']) * $DEG_TO_RAD;
  $latitudeH = sin($latitudeArc * 0.5);
  $latitudeH *= $latitudeH;
  $lontitudeH = sin($longitudeArc * 0.5);
  $lontitudeH *= $lontitudeH;
  $tmp = cos($from['lat']*$DEG_TO_RAD) * cos($to['lat']*$DEG_TO_RAD);
  return 2.0 * asin(sqrt($latitudeH + $tmp*$lontitudeH));

}

function distanceInMeters($from, $to) {
  global $EARTH_RADIUS_IN_METERS;
  return $EARTH_RADIUS_IN_METERS*arcInRadians($from, $to);
}

/**
  * @param $center - geographical point where is the center of the arc e.g array('lat' => 49.1, 'lon' => 16.2)
  * @param $start - geographical point where to start arc
  * @param $end - geographicl point where to end arc
  * @param $cw - clock wise 
  */


function insertArcII($center, $start, $end, $cw) {
  global $M_TO_NM;
  $r = 0.5 * ( distToAngle(distanceInMeters($start, $center) * $M_TO_NM)
    + distToAngle(distanceInMeters($end, $center) * $M_TO_NM) );

  $a1 = angleRad($center, $start);
  $a2 = angleRad($center, $end);

  $result = array();
  array_push($result, $start);

  $result = array_merge($result, insertMidArc($center, $a1, $a2, $cw, $r));
  
  array_push($result, $end);

  return $result;
}


function insertMidArc($center, $from, $to, $cw, $r) {
  global $ARC_GRANULARITY;
  $result = array();
  if ($from < 0) {
    $from += 2*M_PI;
  }
  if ($cw) {
    if ($to < $from) {
      $to += 2*M_PI;
    }
    for ($angle = $from + $ARC_GRANULARITY; $angle < $to; $angle += $ARC_GRANULARITY) {
       array_push($result, computeArcPoint($center['lat'], $center['lon'], $r, $angle));
    }
  } else {
    if ($from < $to) {
      $from += 2*M_PI;
    }
//echo rad2deg($from). " ". rad2deg($to). "<br/>";
    for ($angle = $from - $ARC_GRANULARITY; $angle > $to; $angle -= $ARC_GRANULARITY) {
//echo rad2deg($angle)."<br/>";
       array_push($result, computeArcPoint($center['lat'], $center['lon'], $r, $angle));
    }
  }
  return $result;
}

function angleRad($center, $poi) {
  global $DEG_TO_RAD;
  $lat = $poi['lat'] - $center['lat'];
  $lon = ($poi['lon'] - $center['lon']) * cos ($poi['lat']*$DEG_TO_RAD);
  return atan2($lon, $lat);
}

?>